<?php
/**
 * Taken and modified from http://www.yiiframework.com/doc/cookbook/38/
 * @author angelystor
 *
 */
class NestedTransactionPDO extends PDO
{
	// Database drivers that support SAVEPOINTs.
    protected static $savepointTransactions = array("pgsql", "mysql");

    // The current transaction level.
    protected $transLevel = 0;

    protected function nestable()
    {
        return in_array($this->getAttribute(PDO::ATTR_DRIVER_NAME),
                        self::$savepointTransactions);
    }

    public function transLevel()
    {
    	return $this->transLevel;
    }

    public function beginTransaction()
    {
    	//Yii::trace("BEGIN TRANSACTION ".$this->transLevel);
        if(!$this->nestable() || $this->transLevel == 0)
        {
            parent::beginTransaction();
        }
        else
        {
        	//Yii::trace("exec SAVEPOINT LEVEL{$this->transLevel}");
            $this->exec("SAVEPOINT LEVEL{$this->transLevel}");
        }

        $this->transLevel++;
    }

    public function commit()
    {
        //Yii::trace("END TRANSACTION ".$this->transLevel);
        $this->transLevel--;

        if(!$this->nestable() || $this->transLevel == 0)
        {
            parent::commit();
        }
        else
        {
        	//Yii::trace("exec RELEASE SAVEPOINT LEVEL{$this->transLevel}");
            $this->exec("RELEASE SAVEPOINT LEVEL{$this->transLevel}");
        }
    }

    /*
    public function rollBack()
    {
        $this->transLevel--;

        if(!$this->nestable() || $this->transLevel == 0)
        {
            parent::rollBack();
        }
        else
        {
            $this->exec("ROLLBACK TO SAVEPOINT LEVEL{$this->transLevel}");
        }
    }
    */

    /**
	 * We don't need the above method for now because we're not doing savepoint acceptance. Either the whole thing passes, or it fails
     */
    public function rollBack()
    {
    	parent::rollBack();
    }

}
?>